#include <SDL.h>
#include <sys/types.h>
#include <sys/select.h>
#include <stdlib.h>
#include <linux/input.h>
#include <getopt.h>
#include <string.h>
#include "xenfb.h"

struct data
{
	SDL_Surface *dst;
	SDL_Surface *src;
};

void sdl_update(struct xenfb *xenfb, int x, int y, int width, int height)
{
	struct data *data = xenfb->user_data;
	SDL_Rect r = { x, y, width, height };
	SDL_BlitSurface(data->src, &r, data->dst, &r);
	SDL_UpdateRect(data->dst, x, y, width, height);
}

int sdl2linux[1024] = {
	[SDLK_a] = KEY_A,
	[SDLK_b] = KEY_B,
	[SDLK_c] = KEY_C,
	[SDLK_d] = KEY_D,
	[SDLK_e] = KEY_E,
	[SDLK_f] = KEY_F,
	[SDLK_g] = KEY_G,
	[SDLK_h] = KEY_H,
	[SDLK_i] = KEY_I,
	[SDLK_j] = KEY_J,
	[SDLK_k] = KEY_K,
	[SDLK_l] = KEY_L,
	[SDLK_m] = KEY_M,
	[SDLK_n] = KEY_N,
	[SDLK_o] = KEY_O,
	[SDLK_p] = KEY_P,
	[SDLK_q] = KEY_Q,
	[SDLK_r] = KEY_R,
	[SDLK_s] = KEY_S,
	[SDLK_t] = KEY_T,
	[SDLK_u] = KEY_U,
	[SDLK_v] = KEY_V,
	[SDLK_w] = KEY_W,
	[SDLK_x] = KEY_X,
	[SDLK_y] = KEY_Y,
	[SDLK_z] = KEY_Z,
	[SDLK_0] = KEY_0,
	[SDLK_1] = KEY_1,
	[SDLK_2] = KEY_2,
	[SDLK_3] = KEY_3,
	[SDLK_4] = KEY_4,
	[SDLK_5] = KEY_5,
	[SDLK_6] = KEY_6,
	[SDLK_7] = KEY_7,
	[SDLK_8] = KEY_8,
	[SDLK_9] = KEY_9,
	[SDLK_SPACE] = KEY_SPACE,
	[SDLK_RETURN] = KEY_ENTER,
	[SDLK_PERIOD] = KEY_DOT,
	[SDLK_SLASH] = KEY_SLASH,
	[SDLK_BACKSPACE] = KEY_BACKSPACE,
	[SDLK_TAB] = KEY_TAB,
	[SDLK_LSHIFT] = KEY_LEFTSHIFT,
	[SDLK_RSHIFT] = KEY_RIGHTSHIFT,
	[SDLK_LALT] = KEY_LEFTALT,
	[SDLK_RALT] = KEY_RIGHTALT,
};

static struct option options[] = {
    { "domid", 1, NULL, 'd' },
    { "title", 1, NULL, 't' },
};

int main(int argc, char **argv)
{
	struct xenfb *xenfb;
	int fd;
	int domid = -1;
        char * title = NULL;
	fd_set readfds;
	struct data data;
	SDL_Rect r;
	struct timeval tv = { 0, 500 };
	int do_quit = 0;
        int opt;

	while ((opt = getopt_long(argc, argv, "d:t:", options,
				  NULL)) != -1) {
		switch (opt) {
                case 'd':
                    domid = strtol(optarg, NULL, 10);
                    break;
                case 't':
                    title = strdup(optarg);
                    break;
                }
        }
        if (optind != argc) {
            fprintf(stderr, "Invalid options!\n");
            exit(1);
        }
        if (domid == -1) {
            fprintf(stderr, "Domain ID must be specified!\n");
            exit(1);
        }

	xenfb = xenfb_new();
	if (xenfb == NULL)
		return 1;

	if (!xenfb_attach_dom(xenfb, domid))
		return 1;

	SDL_Init(SDL_INIT_VIDEO);

	fd = xenfb_get_fileno(xenfb);

	data.dst = SDL_SetVideoMode(xenfb->width, xenfb->height, xenfb->depth,
				    SDL_SWSURFACE);

	data.src = SDL_CreateRGBSurfaceFrom(xenfb->pixels,
					    xenfb->width, xenfb->height,
					    xenfb->depth, xenfb->row_stride,
					    0xFF0000, 0xFF00, 0xFF, 0);

        if (title == NULL)
            title = strdup("xen-sdlfb");
        SDL_WM_SetCaption(title, title);

	r.x = r.y = 0;
	r.w = xenfb->width;
	r.h = xenfb->height;
	SDL_BlitSurface(data.src, &r, data.dst, &r);
	SDL_UpdateRect(data.dst, 0, 0, xenfb->width, xenfb->height);

	xenfb->update = sdl_update;
	xenfb->user_data = &data;

	FD_ZERO(&readfds);
	FD_SET(fd, &readfds);

	SDL_ShowCursor(0);

	while (!do_quit && select(fd + 1, &readfds, NULL, NULL, &tv) != -1) {
		SDL_Event event;

		while (SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_KEYDOWN:
			case SDL_KEYUP:
				xenfb_send_key(xenfb,
					       event.type == SDL_KEYDOWN,
					       sdl2linux[event.key.keysym.sym]);
				break;
			case SDL_MOUSEMOTION: {
				int x, y;
				Uint8 button;

				button = SDL_GetRelativeMouseState(&x, &y);
				xenfb_send_motion(xenfb, x, y);
			}	break;
			case SDL_MOUSEBUTTONDOWN:
			case SDL_MOUSEBUTTONUP:
				xenfb_send_button(xenfb,
						  event.type==SDL_MOUSEBUTTONDOWN,
						  3 - event.button.button);
				break;
			case SDL_QUIT:
				do_quit = 1;
				break;
			}
		}
		if (FD_ISSET(fd, &readfds))
			xenfb_on_incoming(xenfb);

		FD_ZERO(&readfds);
		FD_SET(fd, &readfds);

		tv = (struct timeval){0, 500};
	}

	xenfb_delete(xenfb);

	SDL_Quit();

	return 0;
}
